{% extends "base.html" %}

{% block title %}
	&raquo; Gas Supplies &raquo; {{g_supply.id}} &raquo; Notes
{% endblock %}

{% block nav %}
	<a href="/g_supplies">Gas Supplies</a>  &raquo;
	<a href="/g_supplies/{{g_supply.id}}">{{g_supply.id}}</a> &raquo; Notes
	[<a href="/g_supplies/{{g_supply.id}}/notes/add">add</a>]
{% endblock %}

{% block content %}
	<table>
		<thead>
			<tr>
				<th>Edit</th>
				<th>Category</th>
				<th>Importance</th>
				<th>Body</th>
			</tr>
		</thead>
		<tbody>
			{% for note in g_supply_note.notes %}
				<tr>
					<td>
						[<a href="/g_supplies/{{g_supply.id}}/notes/{{loop.index0}}/edit">edit</a>]
					</td>
					<td>{{note.category}}</td>
					<td>
						{% if note.is_important %}
							High
						{% else %}
							Low
						{% endif %}
					</td>
					<td>
						<pre style=" white-space: pre-wrap;">{{note.body}}</pre>
					</td>
				</tr>
			{% endfor %}
		</tbody>
	</table>
{% endblock %}
