{% extends "base.html" %}

{% block title %}
	&raquo; Report Runs
{% endblock %}

{% block nav %}
	Report Runs
{% endblock %}

{% block content %}
	<table class="sticky">
		<thead>
			<tr>
				<th>View</th>
				<th>Date Created</th>
				<th>Created By</th>
				<th>Name</th>
				<th>Title</th>
				<th>State</th>
			</tr>
		</thead>
		<tbody>
			{% for run in runs %}
				<tr>
					<td>
						<a href="/report_runs/{{run.id}}">View</a>
					</td>
					<td>{{run.date_created|hh_format}}</td>
					<td>{{run.creator}}</td>
					<td>{{run.name}}</td>
					<td>{{run.title}}</td>
					<td>{{run.state}}</td>
				</tr>
			{% endfor %}
		</tbody>
	</table>
{% endblock %}
